# Reseda version 1.0.0
# ResedaInterpreter.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Scripts

# 1) Imported modules
# -------------------

from S2MLCore import *

from ResedaSimulator import *

# 2) Creator
# ----------

class ResedaInterpreter(S2MLInterpreter):
  def __init__(self, engine):
    S2MLInterpreter.__init__(self, engine)
    # self._simulator = self._engine.GetSimulator()

  def InitializeEnvironmentVariables(self):
    self.SetValueOfEnvironmentVariable("seed", S2MLConstant.NewInteger(12345))
    self.SetValueOfEnvironmentVariable("number-of-tries", S2MLConstant.NewInteger(0))
    self.SetValueOfEnvironmentVariable("output", S2MLConstant.NewString("results.txt"))
    self.SetValueOfEnvironmentVariable("mode", S2MLConstant.NewSymbol("write"))

# 3) Scripts
# ----------

  def _ParseCommand(self):
    commandName = self.LookUpArgument()
    if commandName=="compute":
      commandName = self.GetArgument()
      self._errors += self._ParseCommandCompute()
    else:
      S2MLInterpreter._ParseCommand(self)

  def _ParseCommandCompute(self):
    # verifies that there is an instantiated model and a target block
    targetBlock = self._engine.LookForTargetBlock()
    if targetBlock==None:
      return 1
    what = self.GetArgument()
    if what=="shortest-path":
      return self._ParseCommandComputeShortestPath()
    if what=="dates":
      return self._ParseCommandComputeDates()
    if what=="maximum-flow":
      return self._ParseCommandComputeMaximumFlow()
    if what=="values":
      return self._ParseCommandComputeValues(True, True, True)
    if what=="parameters":
      return self._ParseCommandComputeValues(True, False, False)
    if what=="attributes":
      return self._ParseCommandComputeValues(False, True, False)
    if what=="properties":
      return self._ParseCommandComputeValues(False, False, True)
    self._RaiseError("line %d\nSyntax error\n" % self._lineNumber)
    return 1

  # compute shortest-path source-node target-node attribute-name fileName [mode=(append|write)]
  def _ParseCommandComputeShortestPath(self):
    sourceNodeName = self.GetArgument()
    if sourceNodeName==None:
      self._RaiseError("line %d\nThe name of the source node was expected\n" % self._lineNumber)
      return 1
    targetNodeName = self.GetArgument()
    if targetNodeName==None:
      self._RaiseError("line %d\nThe name of the target node was expected\n" % self._lineNumber)
      return 1
    attributeName = self.GetArgument()
    if attributeName==None:
      self._RaiseError("line %d\nThe name of the attribute defining the length of edges was expected\n" % self._lineNumber)
      return 1
    fileName = self.GetArgument()
    if fileName==None:
      self._RaiseError("line %d\nA file name was expected\n" % self._lineNumber)
      return 1
    mode = self.GetModeOption("w")
    if mode==None:
      return 1
    matrix = self._engine.ComputeShortestPathWithDijskraAlgorithm(sourceNodeName, targetNodeName, attributeName)
    if matrix==None:
      return 1
    filePath = self._engine.GetFilePath(fileName)
    error = self._printer.ExportMatrix(matrix, filePath, mode)
    return error

  # compute dates source-node target-node attribute-name fileName [mode=(append|write)]
  def _ParseCommandComputeDates(self):
    sourceNodeName = self.GetArgument()
    if sourceNodeName==None:
      self._RaiseError("line %d\nThe name of the source node was expected\n" % self._lineNumber)
      return 1
    targetNodeName = self.GetArgument()
    if targetNodeName==None:
      self._RaiseError("line %d\nThe name of the target node was expected\n" % self._lineNumber)
      return 1
    attributeName = self.GetArgument()
    if attributeName==None:
      self._RaiseError("line %d\nThe name of the attribute defining the length of edges was expected\n" % self._lineNumber)
      return 1
    fileName = self.GetArgument()
    if fileName==None:
      self._RaiseError("line %d\nA file name was expected\n" % self._lineNumber)
      return 1
    mode = self.GetModeOption("w")
    if mode==None:
      return 1
    matrix = self._engine.ComputeDatesOfPERTDiagram(sourceNodeName, targetNodeName, attributeName)
    if matrix==None:
      return 1
    filePath = self._engine.GetFilePath(fileName)
    error = self._printer.ExportMatrix(matrix, filePath, mode)
    return error

  # compute maximum-flow source-node target-node attribute-name fileName [mode=(append|write)]
  def _ParseCommandComputeMaximumFlow(self):
    sourceNodeName = self.GetArgument()
    if sourceNodeName==None:
      self._RaiseError("line %d\nThe name of the source node was expected\n" % self._lineNumber)
      return 1
    targetNodeName = self.GetArgument()
    if targetNodeName==None:
      self._RaiseError("line %d\nThe name of the target node was expected\n" % self._lineNumber)
      return 1
    attributeName = self.GetArgument()
    if attributeName==None:
      self._RaiseError("line %d\nThe name of the attribute defining the length of edges was expected\n" % self._lineNumber)
      return 1
    fileName = self.GetArgument()
    if fileName==None:
      self._RaiseError("line %d\nA file name was expected\n" % self._lineNumber)
      return 1
    mode = self.GetModeOption("w")
    if mode==None:
      return 1
    matrix = self._engine.ComputeMaximumFlowWithFordFulkersonAlgorithm(sourceNodeName, targetNodeName, attributeName)
    if matrix==None:
      return 1
    filePath = self._engine.GetFilePath(fileName)
    error = self._printer.ExportMatrix(matrix, filePath, mode)
    return error

  def _ParseCommandComputeValues(self, printParameters, printAttributes, printProperties):
    missionTimes = [0.0]
    while True:
      argument = self.LookUpArgument()
      if argument==None:
        break
      if argument=="output":
        stringValue = self.GetStringOption("output", "results.txt")
        if stringValue==None:
          return 1
        self._engine.SetOutputFileName(stringValue)
      elif argument=="mode":
        stringValue = self.GetModeOption("write")
        if stringValue==None:
          return 1
        self._engine.SetOutputFileOpeningMode(stringValue)
      elif argument=="mission-time" or argument=="mission-times" :
        missionTimes = self.GetMissionTimeOption(missionTimes)
        if missionTimes==None:
          return 1
      else:
        self._engine.RaiseError("Incorrect argument '{0:s}' of command 'compute values'".format(argument))
        return 1
    self._engine.CalculateValuesOfElements(printParameters, printAttributes, printProperties)
    return 0

  def _ParseCalculationArguments(self):
    self._engine.SetSeed(self.GetValueOfEnvironmentVariable("seed").GetValue())
    self._engine.SetNumberOfTries(self.GetValueOfEnvironmentVariable("number-of-tries").GetValue())
    self._engine.SetOutputFileName(self.GetValueOfEnvironmentVariable("output").GetValue())
    self._engine.SetOutputFileOpeningMode(self.GetValueOfEnvironmentVariable("mode").GetValue())
    self._engine.ResetMissionTimes()
    while True:
      argument = self.LookUpArgument()
      if argument==None:
        break
      if argument=="output":
        stringValue = self.GetStringOption("output", "results.txt")
        if stringValue==None:
          return 1
        self._engine.SetOutputFileName(stringValue)
      elif argument=="mode":
        stringValue = self.GetModeOption("write")
        if stringValue==None:
          return 1
        self._engine.SetOutputFileOpeningMode(stringValue)
      elif argument=="mission-time" or argument=="mission-times" :
        missionTimes = self.GetMissionTimeOption(missionTimes)
        if missionTimes==None:
          return 1
        self._engine.SetMissionTimes(missionTimes)
      else:
        self._engine.RaiseError("Incorrect argument '{0:s}' of command 'compute values'".format(argument))
        return 1
    return 0
