# S2MLCore version 1.2.0
# S2MLEnvironmentManager.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Access methods
# 4) Variables
# 5) Indicator profiles

# 1) Imported modules
# -------------------

from S2MLManagedValue import *
from S2MLStatistics import *
from S2MLModels import *

# 2) Creator
# ----------

class S2MLEnvironmentManager:
  DEFAULT_INDICATOR_PROFILE_NAME = "profile1"

  def __init__(self, engine):
    from S2MLEngine import S2MLEngine
    self._engine = engine
    self._variables = dict()
    self._profiles = dict()
    self._printer = None

# 3) Access methods
# -----------------

  def CreateTools(self):
    self._printer = self._engine.GetScriptPrinter()

  def GetEngine(self):
    return self._engine

  def _RaiseError(self, message):
    self._engine.RaiseError(message)

  def Initialize(self):
    self.SetValueOfEnvironmentVariable("seed", S2MLManagedValue.NewInteger(12345))
    self.SetValueOfEnvironmentVariable("number-of-tries", S2MLManagedValue.NewInteger(0))
    # self.SetValueOfEnvironmentVariable("output", S2MLManagedValue.NewNone())
    self.SetValueOfEnvironmentVariable("mode", S2MLManagedValue.NewIdentifier("write"))
    self.SetValueOfEnvironmentVariable("mission-times", S2MLManagedValue.NewList([S2MLManagedValue.NewReal(0.0)]))
    profile = self._NewIndicatorProfile(S2MLEnvironmentManager.DEFAULT_INDICATOR_PROFILE_NAME)
    profile.SetMean(True)
    profile.SetStandardDeviation(True)
    profile.SetConfidenceInterval90(False)
    profile.SetConfidenceInterval95(True)
    profile.SetConfidenceInterval99(False)
    profile.SetExtrema(True)
    profile.SetQuantiles(False)
    profile.SetDistribution(False)
    profile.SetCumulativeDistribution(False)
    profile.SetBins(False)
    profile.SetBootstrapping(False)

# 4) Variables
# ------------

  def GetVariableIdentifiers(self):
    return self._variables

  def GetValueOfEnvironmentVariable(self, identifier, defaultValue = None):
    return self._variables.get(identifier, defaultValue)

  def SetValueOfEnvironmentVariable(self, identifier, value):
    self._variables[identifier] = value
    return 0

  def ResetValueOfEnvironmentVariable(self, identifier):
    if identifier in self._variables:
      self._variables.pop(identifier)
    return 0

  def ExportEnvironmentVariables(self, fileName, mode="w"):
    filePath = self._engine.GetFilePath(fileName)
    error = self._printer.ExportEnvironmentVariables(self.GetEnvironmentVariables(), filePath, mode)
    return error

  def PrintEnvironmentVariables(self, outputFile):
    self._printer.PrintEnvironmentVariables(self.GetEnvironmentVariables(), outputFile)
    return 0

  def ExportEnvironmentVariable(self, identifier, fileName, mode="w"):
    value = self.GetValueOfEnvironmentVariable(identifier)
    if value==None:
      self.RaiseError("No variable {0:s}".format(identifier))
      return 1
    filePath = self.GetFilePath(fileName)
    error = self._printer.ExportEnvironmentVariable(identifier, value, filePath, mode)
    return error

  def PrintEnvironmentVariable(self, identifier, outputFile):
    value = self.GetValueOfEnvironmentVariable(identifier)
    if value==None:
      self.RaiseError("No variable {0:s}".format(identifier))
      return 1
    self._printer.PrintEnvironmentVariable(identifier, value, outputFile)
    return 0


# 5) Indicator profiles
# ---------------------

  def GetIndicatorProfile(self, name):
    return self._profiles.get(name, None)

  def GetFirstIndicatorProfile(self):
    if self.GetNumberOfProfiles()==0:
      return None
    return self._profiles.values()[0]

  def GetIndicatorProfiles(self):
    return self._profiles.values()

  def GetIndicatorProfileNames(self):
    return self._profiles.keys()

  def GetNumberOfIndicatorProfiles(self):
    return len(self._profiles)

  def _NewIndicatorProfile(self, name):
    profile = S2MLIndicatorProfile(name)
    self._profiles[name] = profile
    return profile

  def _CloneIndicatorProfile(self, profile, name):
    clone = profile.Clone(name)
    self._profiles[name] = clone
    return clone

  def _DeleteIndicatorProfile(self, profile):
    if profile.GetName() in self._profiles:
      self._profiles.pop(profile.GetName())

  def GetDefaultIndicatorProfile(self):
    profile = self.GetIndicatorProfile(S2MLEnvironmentManager.DEFAULT_INDICATOR_PROFILE_NAME)
    if profile!=None:
      return profile
    profile = self.GetFirstIndicatorProfile()
    if profile!=None:
      return profile
    return self.GetIndicatorProfile(S2MLEnvironmentManager.DEFAULT_INDICATOR_PROFILE_NAME)

  def NewIndicatorProfile(self, sourceName):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile!=None:
      self.RaiseError("Indicator profile {0:s} already exist".format(sourceName))
      return 1
    self._NewIndicatorProfile(sourceName)
    return 0

  def CloneIndicatorProfile(self, sourceName, targetName):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    targetProfile = self.GetIndicatorProfile(targetName)
    if targetProfile!=None:
      self.RaiseError("Indicator profile {0:s} already exist".format(targetName))
      return 1
    self._CloneIndicatorProfile(sourceProfile, targetName)
    return 0

  def DeleteIndicatorProfile(self, sourceName):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    self.DeleteProfile(sourceProfile)
    return 0

  def SetIndicatorProfileOptionMean(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetMean(value)
    return 0

  def SetIndicatorProfileOptionStandardDeviation(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetStandardDeviation(value)
    return 0

  def SetIndicatorProfileOptionConfidenceInterval90(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetConfidenceInterval90(value)
    return 0

  def SetIndicatorProfileOptionConfidenceInterval95(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetConfidenceInterval95(value)
    return 0

  def SetIndicatorProfileOptionConfidenceInterval99(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetConfidenceInterval99(value)
    return 0

  def SetIndicatorProfileOptionExtrema(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetExtrema(value)
    return 0

  def SetIndicatorProfileOptionQuantiles(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetQuantiles(value)
    return 0

  def SetIndicatorProfileOptionNumberOfQuantiles(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetNumberOfQuantiles(value)
    return 0

  def SetIndicatorProfileOptionDistribution(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetDistribution(value)
    return 0

  def SetIndicatorProfileOptionCumulativeDistribution(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetCumulativeDistribution(value)
    return 0

  def SetIndicatorProfileOptionNumberOfPoints(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetNumberOfPoints(value)
    return 0

  def SetIndicatorProfileOptionBins(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetBins(value)
    return 0

  def SetIndicatorProfileOptionNumberOfBins(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetNumberOfBins(value)
    return 0

  def SetIndicatorProfileOptionNumberOfStoredValues(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetNumberOfStoredValues(value)
    return 0

  def SetIndicatorProfileOptionBootstrapping(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetBootstrapping(value)
    return 0
 
  def SetIndicatorProfileOptionNumberOfSamples(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetNumberOfSamples(value)
    return 0

  def SetIndicatorProfileOptionSamplingRatio(self, sourceName, value):
    sourceProfile = self.GetIndicatorProfile(sourceName)
    if sourceProfile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    sourceProfile.SetSamplingRatio(value)
    return 0

  def ExportIndicatorProfiles(self, fileName, mode="w"):
    error = self._printer.ExportIndicatorProfiles(self.GetIndicatorProfiles(), fileName, mode)
    return error

  def PrintIndicatorProfiles(self):
    self._printer.PrintIndicatorProfiles(self.GetIndicatorProfiles(), self._interpreter.GetOutputFile())
    return 0

  def ExportIndicatorProfile(self, sourceName, fileName, mode="w"):
    profile = self.GetIndicatorProfile(sourceName)
    if profile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    filePath = self.GetFilePath(fileName)
    error = self._printer.ExportIndicatorProfile(profile, filePath, mode)
    return error

  def PrintIndicatorProfile(self, sourceName):
    profile = self.GetIndicatorProfile(identifier)
    if profile==None:
      self.RaiseError("No indicator profile {0:s}".format(sourceName))
      return 1
    self._printer.PrintIndicatorProfile(profile, self._interpreter.GetOutputFile())
    return 0
