# S2MLCore version 1.2.0
# S2MLModelManager.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Accessors
# 4) Models

# 1) Imported modules
# -------------------

from S2MLModels import *
from S2MLInterface import *

from S2MLModelChecker import *
from S2MLModelFlattener import *
from S2MLModelRewriter import *

# 2) Creator
# ----------

class S2MLModelManager:
  def __init__(self, engine):
    from S2MLEngine import S2MLEngine
    self._engine = engine
    self._modelLexer = None
    self._modelParser = None
    self._modelPrinter = None
    self._modelChecker = None
    self._modelFlattener = None
    self._modelRewriter = None
    self._sourceModel = None
    self._targetModel = None
    self._targetBlock = None
    self._forceFlattening = False

# 3) Accessors
# ------------

  # To be overloaded
  def CreateTools(self):
    self._modelLexer = S2MLModelLexer(self._engine)
    self._modelParser = S2MLModelParser(self._engine)
    self._modelPrinter = S2MLModelPrinter(self._engine)
    self._modelChecker = S2MLModelChecker(self._engine)
    self._modelFlattener = S2MLModelFlattener(self._engine)
    self._modelRewriter = S2MLModelRewriter(self._engine)

  def GetModelLexer(self):
    return self._modelLexer

  def GetModelParser(self):
    return self._modelParser

  def GetModelPrinter(self):
    return self._modelPrinter

  def GetModelChecker(self):
    return self._modelChecker

  def GetModelFlattener(self):
    return self._modelFlattener

  def GetModelRewriter(self):
    return self._modelFlattener

  def RaiseError(self, message):
    self._engine.RaiseError(message)

# 4) Models
# ---------

  # To be overloaded
  def NewModel(self):
    return S2MLModel()

  def ResetModel(self):
    self._sourceModel = self.NewModel()
    self._targetModel = None
    self._targetBlock = None
    return 0

  def GetSourceModel(self):
    return self._sourceModel

  def SetSourceModel(self, sourceModel):
    self._sourceModel = sourceModel

  def GetTargetModel(self):
    return self._targetModel

  def SetTargetModel(self, targetModel):
    self._targetModel = targetModel

  def GetTargetBlock(self):
    return self._targetBlock

  def SetTargetBlock(self, targetBlock):
    self._targetBlock = targetBlock

  def ImportModel(self, fileName):
    if self._sourceModel==None:
      self.ResetModel()
    self._targetModel = None
    self._targetBlock = None
    filePath = self._engine.GetFilePath(fileName)
    error = self._modelParser.ImportFile(self._sourceModel, filePath.as_posix())
    if error!=0:
      self.ResetModel()
    return error

  def ExportModel(self, fileName, mode="w"):
    if self._sourceModel==None:
      self.RaiseError("No model to export")
      return 1
    filePath = self._engine.GetFilePath(fileName)
    error = self._modelPrinter.ExportModel(self._sourceModel, filePath.as_posix(), mode)
    return error

  def PrintModel(self, outputFile):
    if self._sourceModel==None:
      self.RaiseError("No model to export")
      return 1
    self._modelPrinter.PrintModel(self._sourceModel, outputFile)
    return 0

  def InstantiateModel(self, flattening = True):
    if self._sourceModel==None:
      self.RaiseError("No model to check")
      return 1
    if flattening or self._forceFlattening:
      self._targetModel = self._modelFlattener.InstantiateAndFlattenModel(self._sourceModel)
    else:
      self._targetModel = self._modelFlattener.InstantiateModel(self._sourceModel)
    self._targetBlock = None
    if self._targetModel==None:
      return 1
    error = self._modelChecker.CheckModel(self._targetModel)
    if error!=0:
      self._targetModel = None
      return error
    error = self._modelRewriter.PostProcessModel(self._targetModel)
    if error!=0:
      self._targetModel = None
      return error
    self._targetBlock = self._targetModel.GetBlock(0)
    return 0

  def ExportInstantiatedModel(self, fileName, mode="w"):
    if self._targetModel==None:
      self.RaiseError("No model to export")
      return 1
    filePath = self._engine.GetFilePath(fileName)
    self._modelPrinter.ExportModel(self._targetModel, filePath, mode)
    return 0

  def PrintInstantiatedModel(self, outputFile):
    if self._targetModel==None:
      self.RaiseError("No model to export")
      return 1
    self._modelPrinter.PrintModel(self._targetModel, outputFile)
    return 0

  def GetTargetBlock(self):
    return self._targetBlock

  def SetTargetBlock(self, blockName):
    if self._targetModel==None:
      self.RaiseError("No instantiated model")
      return 1
    self._targetBlock = self._targetModel.LookForBlock(blockName)
    if self._targetBlock==None:
      self.RaiseError("No block {0:s} in instantiated model".format(blockName))
      return 1
    return 0

  def ResetTargetBlock(self):
    self._targetBlock = None
    return 0

  def LookForTargetBlock(self, targetBlockName):
    if self._targetModel==None:
      self.RaiseError("No instantiated model")
      return None
    self._targetBlock = self._targetModel.LookForBlock(targetBlockName)
    if self._targetBlock==None:
      self.RaiseError("No block {0:s}".format(targetBlockName))
      return None
    return self._targetBlock

  def SelectForTargetBlock(self):
    if self._targetModel==None:
      self.RaiseError("No instantiated model")
      return None
    if self._targetBlock!=None:
      return self._targetBlock
    topBlocks = []
    for element in self._targetModel.GetElements():
      if S2MLType.IsBlock(element):
        topBlocks.append(element)
    if len(topBlocks)==0:
      self.RaiseError("No target block")
      return None
    if len(topBlocks)>1:
      self.RaiseError("Several candidate target blocks")
      return None
    self._targetBlock = topBlocks[0]
    return self._targetBlock
