# S2MLCore version 1.2.0
# S2MLScriptChecker.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Scripts
# 4) Builtin commands
#  4.1) compute
#  4.2) load
#  4.3) flatten and instantiate
#  4.4) print
#  4.5) reset
#  4.6) set
#  4.7) profile
# 5) Arguments
# 6) Options

# 1) Imported modules
# -------------------

from S2MLManagedValue import *
from S2MLCommand import *

# 2) Creator
# ----------

class S2MLScriptChecker:
  def __init__(self, engine):
    from S2MLEngine import S2MLEngine
    self._engine = engine

# 3) Scripts
# ----------

  def RaiseError(self, message):
    self._errors += 1
    self._engine.RaiseError(message)

  def RaiseErrorInCommand(self, command, message):
    self._errors += 1
    self._engine.RaiseErrorInCommand(command, message)

  def CheckScript(self, script):
    self._errors = 0
    for command in script.GetCommands():
      self.CheckCommand(command)
    return self._errors

  def CheckCommand(self, command):
    if command.GetType()==S2MLCommand.SCRIPT:
      self.CheckScript(command)
    elif command.GetType()==S2MLCommand.BUILTIN:
      self.CheckBuiltInCommand(command)

  def CheckBuiltInCommand(self, command):
    commandName = command.GetName()
    if commandName=="compute":
      self._errors += self.CheckCommandCompute(command)
    elif commandName=="instantiate":
      self._errors += self.CheckCommandInstantiate(command)
    elif commandName=="flatten":
      self._errors += self.CheckCommandFlatten(command)
    elif commandName=="load":
      self._errors += self.CheckCommandLoad(command)
    elif commandName=="print":
      self._errors += self.CheckCommandPrint(command)
    elif commandName=="reset":
      self._errors += self.CheckCommandReset(command)
    elif commandName=="set":
      self._errors += self.CheckCommandSet(command)
    elif commandName=="profile":
      self._errors += self.CheckCommandProfile(command)
    else:
      self.RaiseErrorInCommand("Unknow command {1:s}\n".format(commandName))

# 4) Builtin commands
# ------------------

# 4.1) compute
# ------------

  def CheckCommandCompute(self, command):
    self.RaiseErrorInCommand(command, "Not implemented\n")
    return 1

# 4.2) load
# ---------

  # load model fileName
  # load script fileName
  def CheckCommandLoad(self, command):
    error = self.CheckNumberOfArgumentsOfCommand(command, 2)
    if error!=0:
      return 1
    what = self.GetCommandArgumentAsIdentifier(command, 0)
    if what==None:
      return 1
    if what!="model" and what!="script":
      self.RaiseErrorInCommand(command, "Incorrect argument {0:s}\n".format(what))
      return 1
    fileName = self.GetCommandArgumentAsString(command, 1)
    if fileName==None:
      self.RaiseErrorInCommand(command, "Incorrect argument 2, a file name was expected\n")
      return 1
    return 0

# 4.3) instantiate and flatten
# ----------------------------

  # instantiate model
  def CheckCommandInstantiate(self, command):
    error = self.CheckNumberOfArgumentsOfCommand(command, 1)
    if error!=0:
      return 1
    what = self.GetCommandArgumentAsIdentifier(command, 0)
    if what==None:
      return 1
    if what!="model":
      self.RaiseErrorInCommand(command, "Incorrect argument {0:s}\n".format(what))
      return 1
    return 0

  # flatten model
  def CheckCommandFlatten(self, command):
    error = self.CheckNumberOfArgumentsOfCommand(command, 1)
    if error!=0:
      return 1
    what = self.GetCommandArgumentAsIdentifier(command, 0)
    if what==None:
      return 1
    if what!="model":
      self.RaiseErrorInCommand(command, "Incorrect argument {0:s}\n".format(what))
      return 1
    return 0

# 4.4) print
# ----------

  # print model [output=fileName mode=(append|write)]
  # print instantiated-model [output=fileName mode=(append|write)]
  # print variables [output=fileName mode=(append|write)]
  # print variable variableName [output=fileName mode=(append|write)]
  # print profiles [output=fileName mode=(append|write)]
  # print profile profileName [output=fileName mode=(append|write)]
  def CheckCommandPrint(self, command):
    what = self.GetCommandArgumentAsIdentifier(command, 0)
    if what==None:
      return 1
    if what=="model" or what=="instantiated-model" or what=="variables" or what=="profiles":
      error = self.CheckNumberOfArgumentsOfCommand(command, 1)
      if error!=0:
        return 1
      error = self.CheckOptions(command)
      if error!=0:
        return 1
      return 0
    if what=="variable" or what=="profile":
      error = self.CheckNumberOfArgumentsOfCommand(command, 2)
      if error!=0:
        return 1
      name = self.GetCommandArgumentAsIdentifier(command, 1)
      if name==None:
        return 1
      error = self.CheckOptions(command)
      if error!=0:
        return 1
      return 0
    self.RaiseErrorInCommand(command, "Incorrect argument {0:s}\n".format(what))
    return 1

# 4.5) reset
# ----------

  # reset model
  def CheckCommandReset(self, command):
    error = self.CheckNumberOfArgumentsOfCommand(command, 1)
    if error!=0:
      return 1
    what = self.GetCommandArgumentAsIdentifier(command, 0)
    if what==None:
      return 1
    if what!="model":
      self.RaiseErrorInCommand(command, "Incorrect argument {0:s}\n".format(what))
      return 1
    return 0

# 4.6) set
# --------

  # set block blockName
  def CheckCommandSet(self, command):
    what = self.GetCommandArgumentAsIdentifier(command, 0)
    if what==None:
      return 1
    if what=="target-block":
      error = self.CheckNumberOfArgumentsOfCommand(command, 2)
      if error!=0:
        return 1
      name = self.GetCommandArgumentAsIdentifier(command, 1)
      if name==None:
        return 1
      return 0
    error = self.CheckNumberOfArgumentsOfCommand(command, 2)
    if error!=0:
      return 1
    return 0

# 4.7) profile
# ------------

  def CheckCommandProfile(self, command):
    what = self.GetCommandArgumentAsIdentifier(command, 0)
    if what==None:
      return 1
    if what=="new":
      error = self.CheckNumberOfArgumentsOfCommand(command, 2)
      if error!=0:
        return 1
      sourceName = self.GetCommandArgumentAsIdentifier(command, 1)
      if sourceName==None:
        return 1
      return 0
    if what=="clone":
      error = self.CheckNumberOfArgumentsOfCommand(command, 3)
      if error!=0:
        return 1
      sourceName = self.GetCommandArgumentAsIdentifier(command, 1)
      if sourceName==None:
        return 1
      targetName = self.GetCommandArgumentAsIdentifier(command, 2)
      if targetName==None:
        return 1
      return 0
    if what=="delete":
      error = self.CheckNumberOfArgumentsOfCommand(command, 2)
      if error!=0:
        return 1
      sourceName = self.GetCommandArgumentAsIdentifier(command, 1)
      if sourceName==None:
        return 1
      return 0
    if what=="set":
      error = self.CheckNumberOfArgumentsOfCommand(command, 4)
      if error!=0:
        return 1
      sourceName = self.GetCommandArgumentAsIdentifier(command, 1)
      if sourceName==None:
        return 1
      argument = self.GetCommandArgumentAsIdentifier(command, 2)
      if argument==None:
        return 1
      if (argument=="mean" or argument=="standard-deviation" or
         argument=="confidence-interval-90" or argument=="confidence-interval-95" or argument=="confidence-interval-99" or
         argument=="extrema" or argument=="quantiles" or argument=="distribution" or argument=="cumulative-distribution" or
         argument=="bins" or argument=="bootstrapping"):
        value = self.GetCommandArgumentAsBoolean(command, 3)
        if value==None:
          return 1
        return 0
      if (argument=="number-of-quantiles" or argument=="number-of-points" or argument=="number-of-bins" or
         argument=="number-of-stored-values" or argument=="number-of-samples"):
        value = self.GetCommandArgumentAsInteger(command, 3)
        if value==None:
          return 1
        return 0
      if argument=="sampling-ratio":
        value = self.GetCommandArgumentAsReal(command, 3)
        if value==None:
          return 1
        return 0
      self.RaiseErrorInCommand(command, "Incorrect argument {0:s}\n".format(argument))
      return 1
    self.RaiseErrorInCommand(command, "Incorrect argument {0:s}\n".format(what))
    return 1

# 5) Arguments
# ------------

  def CheckNumberOfArgumentsOfCommand(self, command, expectedNumber):
    if command.GetNumberOfArguments()!=expectedNumber:
      self.RaiseErrorInCommand(command, "Wrong number of arguments ({0:d} expected)\n".format(expectedNumber))
      return 1
    return 0

  def GetCommandArgumentAsBoolean(self, command, index):
    value = command.GetArgumentAsBoolean(index)
    if value==None:
      self.RaiseErrorInCommand(command, "A Boolean value was expected as argument {0:d}\n".format(index+1))
      return None
    return value

  def GetCommandArgumentAsInteger(self, command, index):
    value = command.GetArgumentAsInteger(index)
    if value==None:
      self.RaiseErrorInCommand(command, "An integer was expected as argument {0:d}\n".format(index+1))
      return None
    return value

  def GetCommandArgumentAsIdentifier(self, command, index):
    identifier = command.GetArgumentAsIdentifier(index)
    if identifier==None:
      self.RaiseErrorInCommand(command, "An identifier was expected as argument {0:d}\n".format(index+1))
      return None
    return identifier

  def GetCommandArgumentAsString(self, command, index):
    string = command.GetArgumentAsString(index)
    if string==None:
      self.RaiseErrorInCommand(command, "A string was expected as argument {0:d}\n".format(index+1))
      return None
    return string

# 6) Options
# ----------

  def CheckOptions(self, command):
    error = 0
    for name in command.GetOptions():
      error += self.CheckOption(command, name)
    return error

  def CheckOption(self, command, name):
    value = command.GetOption(name)
    if value==None:
       return 0
    if name=="output":
      if not value.IsString():
        self.RaiseErrorInCommand(command, "The value of option 'output' must be a string\n")
        return 1
      return 0
    if name=="mode":
      identifier = value.GetValueOfIdentifier()
      if identifier!="write" and identifier!="append" and identifier!="w" and identifier!="a":
        self.RaiseErrorInCommand(command, "The value of option 'mode' must be 'write' or 'append'\n")
        return 1
      return 0
    if name=="seed":
      if not value.IsInteger():
        self.RaiseErrorInCommand(command, "The value of option 'seed' must be an integer\n")
        return 1
      return 0
    if name=="number-of-tries":
      if not value.IsInteger():
        self.RaiseErrorInCommand(command, "The value of option 'number-of-tries' must be an integer\n")
        return 1
      return 0
    if name=="mission-times":
      if not value.IsListOfNumbers():
        self.RaiseErrorInCommand(command, "The value of option 'mission-times' must be a list of reals\n")
        return 1
      return 0
    if name=="profile":
      if not value.IsIdentifier():
        self.RaiseErrorInCommand(command, "The value of option 'profile' must be an identifier of indicator profile\n")
        return 1
      return 0
    self.RaiseErrorInCommand(command, "Unknown option '{0:s}'\n".format(name))
    return 1

